import 'package:flutter/material.dart';

import 'package:syncrow_web/pages/device_managment/all_devices/models/device_status.dart';

class CeilingSensorModel {
  String presenceState;
  int sensitivity;
  String checkingResult;
  int presenceRange;
  int sportsPara;
  String bodyMovement;
  String noBodyTime;
  int maxDistance;
  SpaceTypes spaceType;

  CeilingSensorModel({
    required this.presenceState,
    required this.sensitivity,
    required this.checkingResult,
    required this.presenceRange,
    required this.sportsPara,
    required this.bodyMovement,
    required this.noBodyTime,
    required this.maxDistance,
    required this.spaceType,
  });

  factory CeilingSensorModel.fromJson(List<Status> jsonList) {
    late var presenceState = 'none';
    late var sensitivity = 1;
    late var checkingResult = '';
    var presenceRange = 1;
    var sportsPara = 1;
    var bodyMovement = 'none';
    var noBodyTime = 'none';
    var maxDis = 0;
    var spaceType = SpaceTypes.none;

    try {
      for (final status in jsonList) {
        switch (status.code) {
          case 'presence_state':
            presenceState = status.value ?? 'none';
            break;
          case 'scene':
            spaceType = getSpaceType(status.value ?? 'none');
            break;
          case 'sensitivity':
            sensitivity = status.value is int
                ? status.value
                : int.tryParse(status.value ?? '1') ?? 1;
            break;
          case 'checking_result':
            checkingResult = status.value ?? '';
            break;
          case 'presence_range':
            presenceRange = status.value is int
                ? status.value
                : int.tryParse(status.value ?? '0') ?? 0;
            break;
          case 'sports_para':
            sportsPara = status.value is int
                ? status.value
                : int.tryParse(status.value ?? '0') ?? 0;
            break;
          case 'body_movement':
            bodyMovement = status.value ?? '';
            break;
          case 'nobody_time':
            noBodyTime = status.value ?? 'none';
            break;
          case 'moving_max_dis':
            maxDis = status.value is int
                ? status.value
                : int.tryParse(status.value ?? '0') ?? 0;
            break;
        }
      }
    } catch (e) {
      debugPrint(e.toString());
    }

    return CeilingSensorModel(
      presenceState: presenceState,
      sensitivity: sensitivity,
      checkingResult: checkingResult,
      presenceRange: presenceRange,
      sportsPara: sportsPara,
      bodyMovement: bodyMovement,
      noBodyTime: noBodyTime,
      maxDistance: maxDis,
      spaceType: spaceType,
    );
  }

  CeilingSensorModel copyWith({
    String? presenceState,
    int? sensitivity,
    String? checkingResult,
    int? presenceRange,
    int? sportsPara,
    String? bodyMovement,
    String? noBodyTime,
    int? maxDistance,
    SpaceTypes? spaceType,
  }) {
    return CeilingSensorModel(
      presenceState: presenceState ?? this.presenceState,
      sensitivity: sensitivity ?? this.sensitivity,
      checkingResult: checkingResult ?? this.checkingResult,
      presenceRange: presenceRange ?? this.presenceRange,
      sportsPara: sportsPara ?? this.sportsPara,
      bodyMovement: bodyMovement ?? this.bodyMovement,
      noBodyTime: noBodyTime ?? this.noBodyTime,
      maxDistance: maxDistance ?? this.maxDistance,
      spaceType: spaceType ?? this.spaceType,
    );
  }
}

enum SpaceTypes {
  none,
  parlour,
  area,
  toilet,
  bedroom,
}

SpaceTypes getSpaceType(String value) {
  switch (value) {
    case 'parlour':
      return SpaceTypes.parlour;
    case 'area':
      return SpaceTypes.area;
    case 'toilet':
      return SpaceTypes.toilet;
    case 'bedroom':
      return SpaceTypes.bedroom;
    case 'none':
    default:
      return SpaceTypes.none;
  }
}
