import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:number_pagination/number_pagination.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/add_user_dialog/view/add_user_dialog.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/add_user_dialog/view/delete_user_dialog.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/add_user_dialog/view/edit_user_dialog.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/add_user_dialog/view/popup_menu_filter.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/bloc/user_table_bloc.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/bloc/user_table_event.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/bloc/user_table_state.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/view/creation_date_filter.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/view/de_activate_filter.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/view/name_filter.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/view/user_table.dart';
import 'package:syncrow_web/pages/space_tree/bloc/space_tree_bloc.dart';
import 'package:syncrow_web/pages/space_tree/bloc/space_tree_event.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';
import 'package:syncrow_web/utils/style.dart';

class UsersPage extends StatelessWidget {
  const UsersPage({super.key});

  @override
  Widget build(BuildContext context) {
    final searchController = TextEditingController();

    Widget actionButton(
        {bool isActive = false, required String title, Function()? onTap}) {
      return InkWell(
        onTap: onTap,
        child: Padding(
          padding: const EdgeInsets.only(left: 8, right: 8),
          child: Text(
            title,
            style: Theme.of(context).textTheme.bodySmall?.copyWith(
                  color: isActive == false && title != 'Delete'
                      ? Colors.grey
                      : title == 'Delete'
                          ? ColorsManager.red
                          : ColorsManager.spaceColor,
                  fontWeight: FontWeight.w400,
                ),
          ),
        ),
      );
    }

    Widget status({required String status}) {
      return Center(
        child: Container(
          padding: const EdgeInsets.only(left: 5, right: 5, bottom: 5, top: 5),
          decoration: BoxDecoration(
            borderRadius: const BorderRadius.all(Radius.circular(20)),
            color: status == 'invited'
                ? ColorsManager.invitedOrange.withValues(alpha: 0.5)
                : status == 'active'
                    ? ColorsManager.activeGreen.withValues(alpha: 0.5)
                    : ColorsManager.disabledPink.withValues(alpha: 0.5),
          ),
          child: Padding(
            padding:
                const EdgeInsets.only(left: 10, right: 10, bottom: 5, top: 5),
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Text(
                  status,
                  style: Theme.of(context).textTheme.bodySmall?.copyWith(
                        color: status == 'invited'
                            ? ColorsManager.invitedOrangeText
                            : status == 'active'
                                ? ColorsManager.activeGreenText
                                : ColorsManager.disabledRedText,
                        fontWeight: FontWeight.w400,
                      ),
                ),
              ],
            ),
          ),
        ),
      );
    }

    Widget changeIconStatus(
        {required String userId,
        required String status,
        required Function()? onTap}) {
      return Center(
        child: InkWell(
          onTap: onTap,
          child: Padding(
            padding:
                const EdgeInsets.only(left: 5, right: 5, bottom: 5, top: 5),
            child: SvgPicture.asset(
              status == 'invited'
                  ? Assets.invitedIcon
                  : status == 'active'
                      ? Assets.activeUser
                      : Assets.deActiveUser,
              height: 35,
            ),
          ),
        ),
      );
    }

    return BlocBuilder<UserTableBloc, UserTableState>(
      builder: (context, state) {
        final screenSize = MediaQuery.of(context).size;
        final blocRole = BlocProvider.of<UserTableBloc>(context);
        if (state is UsersLoadingState) {
          return const Center(child: CircularProgressIndicator());
        } else if (state is UsersLoadedState) {
          return Padding(
            padding: const EdgeInsets.all(20),
            child: Align(
              alignment: Alignment.topCenter,
              child: Column(
                children: [
                  Row(
                    children: [
                      Container(
                        decoration: containerDecoration.copyWith(
                          borderRadius: const BorderRadius.all(
                            Radius.circular(20),
                          ),
                        ),
                        width: screenSize.width * 0.4,
                        child: TextFormField(
                          controller: searchController,
                          onChanged: (value) {
                            final bloc = context.read<UserTableBloc>();
                            bloc.add(const FilterClearEvent());
                            bloc.add(SearchUsers(value));
                            if (value == '') {
                              bloc.add(const ChangePage(1));
                            }
                          },
                          style: const TextStyle(color: Colors.black),
                          decoration: textBoxDecoration(radios: 15)!.copyWith(
                            fillColor: ColorsManager.whiteColors,
                            errorStyle: const TextStyle(height: 0),
                            hintStyle: context.textTheme.titleSmall?.copyWith(
                              color: Colors.grey,
                              fontSize: 12,
                            ),
                            hintText: 'Search',
                            suffixIcon: SizedBox(
                              child: SvgPicture.asset(
                                Assets.searchIconUser,
                                fit: BoxFit.none,
                              ),
                            ),
                          ),
                        ),
                      ),
                      const SizedBox(width: 20),
                      InkWell(
                        onTap: () {
                          context.read<SpaceTreeBloc>().add(ClearCachedData());
                          showDialog(
                            context: context,
                            barrierDismissible: false,
                            builder: (BuildContext context) {
                              return const AddNewUserDialog();
                            },
                          ).then((v) {
                            if (v != null) {
                              blocRole.add(const GetUsers());
                            }
                          });
                        },
                        child: Container(
                          decoration: containerWhiteDecoration,
                          width: screenSize.width * 0.18,
                          height: 50,
                          child: const Center(
                            child: Text(
                              'Add New User',
                              style: TextStyle(
                                fontSize: 16,
                                fontWeight: FontWeight.w700,
                                color: ColorsManager.blueColor,
                              ),
                            ),
                          ),
                        ),
                      ),
                    ],
                  ),
                  const SizedBox(height: 20),
                  SizedBox(
                    height: screenSize.height * 0.65,
                    child: DynamicTableScreen(
                      tableSize: screenSize.height * 0.65,
                      onFilter: (columnIndex) {
                        if (columnIndex == 0) {
                          showNameMenu(
                            context: context,
                            isSelected: blocRole.currentSortOrder,
                            aToZTap: () {
                              context
                                  .read<UserTableBloc>()
                                  .add(const SortUsersByNameAsc());
                            },
                            zToATap: () {
                              context
                                  .read<UserTableBloc>()
                                  .add(const SortUsersByNameDesc());
                            },
                          );
                        }
                        if (columnIndex == 2) {
                          final checkboxStates = <String, bool>{
                            for (final item in blocRole.jobTitle)
                              item: blocRole.selectedJobTitles.contains(item),
                          };
                          final overlay = Overlay.of(context)
                              .context
                              .findRenderObject()! as RenderBox;

                          showPopUpFilterMenu(
                            position: RelativeRect.fromLTRB(
                              overlay.size.width / 5.3,
                              240,
                              overlay.size.width / 4,
                              0,
                            ),
                            list: blocRole.jobTitle,
                            context: context,
                            checkboxStates: checkboxStates,
                            isSelected: blocRole.currentSortJopTitle,
                            onOkPressed: () {
                              searchController.clear();
                              blocRole.add(const FilterClearEvent());
                              final selectedItems = checkboxStates.entries
                                  .where((entry) => entry.value)
                                  .map((entry) => entry.key)
                                  .toList();
                              Navigator.of(context).pop();
                              blocRole.add(FilterUsersByJobEvent(
                                selectedJob: selectedItems,
                                sortOrder: blocRole.currentSortJopTitle,
                              ));
                            },
                            onSortAtoZ: (v) {
                              blocRole.currentSortJopTitle = v;
                            },
                            onSortZtoA: (v) {
                              blocRole.currentSortJopTitle = v;
                            },
                          );
                        }

                        if (columnIndex == 3) {
                          final checkboxStates = <String, bool>{
                            for (final item in blocRole.roleTypes)
                              item: blocRole.selectedRoles.contains(item),
                          };
                          final overlay = Overlay.of(context)
                              .context
                              .findRenderObject()! as RenderBox;
                          showPopUpFilterMenu(
                            position: RelativeRect.fromLTRB(
                              overlay.size.width / 4,
                              240,
                              overlay.size.width / 4,
                              0,
                            ),
                            list: blocRole.roleTypes,
                            context: context,
                            checkboxStates: checkboxStates,
                            isSelected: blocRole.currentSortRole,
                            onOkPressed: () {
                              searchController.clear();
                              blocRole.add(const FilterClearEvent());
                              final selectedItems = checkboxStates.entries
                                  .where((entry) => entry.value)
                                  .map((entry) => entry.key)
                                  .toList();
                              Navigator.of(context).pop();
                              context.read<UserTableBloc>().add(
                                  FilterUsersByRoleEvent(
                                      selectedRoles: selectedItems,
                                      sortOrder: blocRole.currentSortRole));
                            },
                            onSortAtoZ: (v) {
                              blocRole.currentSortRole = v;
                            },
                            onSortZtoA: (v) {
                              blocRole.currentSortRole = v;
                            },
                          );
                        }
                        if (columnIndex == 4) {
                          showDateFilterMenu(
                            context: context,
                            isSelected: blocRole.currentSortOrder,
                            aToZTap: () {
                              context
                                  .read<UserTableBloc>()
                                  .add(const DateNewestToOldestEvent());
                            },
                            zToaTap: () {
                              context
                                  .read<UserTableBloc>()
                                  .add(const DateOldestToNewestEvent());
                            },
                          );
                        }
                        if (columnIndex == 6) {
                          final checkboxStates = <String, bool>{
                            for (final item in blocRole.createdBy)
                              item: blocRole.selectedCreatedBy.contains(item),
                          };
                          final overlay = Overlay.of(context)
                              .context
                              .findRenderObject()! as RenderBox;
                          showPopUpFilterMenu(
                            position: RelativeRect.fromLTRB(
                              overlay.size.width / 1,
                              240,
                              overlay.size.width / 4,
                              0,
                            ),
                            list: blocRole.createdBy,
                            context: context,
                            checkboxStates: checkboxStates,
                            isSelected: blocRole.currentSortCreatedBy,
                            onOkPressed: () {
                              searchController.clear();
                              blocRole.add(const FilterClearEvent());
                              final selectedItems = checkboxStates.entries
                                  .where((entry) => entry.value)
                                  .map((entry) => entry.key)
                                  .toList();
                              Navigator.of(context).pop();
                              blocRole.add(FilterUsersByCreatedEvent(
                                  selectedCreatedBy: selectedItems,
                                  sortOrder: blocRole.currentSortCreatedBy));
                            },
                            onSortAtoZ: (v) {
                              blocRole.currentSortCreatedBy = v;
                            },
                            onSortZtoA: (v) {
                              blocRole.currentSortCreatedBy = v;
                            },
                          );
                        }
                        if (columnIndex == 7) {
                          final checkboxStates = <String, bool>{
                            for (final item in blocRole.status)
                              item: blocRole.selectedStatuses.contains(item),
                          };

                          final overlay = Overlay.of(context)
                              .context
                              .findRenderObject()! as RenderBox;
                          showPopUpFilterMenu(
                            position: RelativeRect.fromLTRB(
                              overlay.size.width / 0,
                              240,
                              overlay.size.width / 5,
                              0,
                            ),
                            list: blocRole.status,
                            context: context,
                            checkboxStates: checkboxStates,
                            isSelected: blocRole.currentSortStatus,
                            onOkPressed: () {
                              searchController.clear();
                              blocRole.add(const FilterClearEvent());
                              final selectedItems = checkboxStates.entries
                                  .where((entry) => entry.value)
                                  .map((entry) => entry.key)
                                  .toList();
                              Navigator.of(context).pop();
                              blocRole.add(FilterUsersByDeActevateEvent(
                                  selectedActivate: selectedItems,
                                  sortOrder: blocRole.currentSortStatus));
                            },
                            onSortAtoZ: (v) {
                              blocRole.currentSortStatus = v;
                            },
                            onSortZtoA: (v) {
                              blocRole.currentSortStatus = v;
                            },
                          );
                        }
                        if (columnIndex == 8) {
                          showDeActivateFilterMenu(
                            context: context,
                            isSelected: blocRole.currentSortOrderDate,
                            aToZTap: () {
                              context
                                  .read<UserTableBloc>()
                                  .add(const DateNewestToOldestEvent());
                            },
                            zToaTap: () {
                              context
                                  .read<UserTableBloc>()
                                  .add(const DateOldestToNewestEvent());
                            },
                          );
                        }
                      },
                      titles: const [
                        'Full Name',
                        'Email Address',
                        'Job Title',
                        'Role',
                        'Creation Date',
                        'Creation Time',
                        'Created By',
                        'Status',
                        'De/Activate',
                        'Action'
                      ],
                      rows: state.users.map((user) {
                        return [
                          Text('${user.firstName} ${user.lastName}'),
                          Text(user.email),
                          Text(user.jobTitle),
                          Text(user.roleType ?? ''),
                          Text(user.createdDate ?? ''),
                          Text(user.createdTime ?? ''),
                          Text(user.invitedBy),
                          status(
                            status: user.isEnabled == false
                                ? 'disabled'
                                : user.status,
                          ),
                          changeIconStatus(
                            status: user.isEnabled == false
                                ? 'disabled'
                                : user.status,
                            userId: user.uuid,
                            onTap: user.status != 'invited'
                                ? () {
                                    context.read<UserTableBloc>().add(
                                        ChangeUserStatus(
                                            userId: user.uuid,
                                            newStatus: user.isEnabled == false
                                                ? 'disabled'
                                                : user.status));
                                  }
                                : null,
                          ),
                          Row(
                            children: [
                              if (user.isEnabled != false)
                                actionButton(
                                  isActive: true,
                                  title: 'Edit',
                                  onTap: () {
                                    context
                                        .read<SpaceTreeBloc>()
                                        .add(ClearCachedData());
                                    showDialog(
                                      context: context,
                                      barrierDismissible: false,
                                      builder: (BuildContext context) {
                                        return EditUserDialog(
                                            userId: user.uuid);
                                      },
                                    ).then((v) {
                                      if (v != null) {
                                        if (v != null) {
                                          blocRole.add(const GetUsers());
                                        }
                                      }
                                    });
                                  },
                                )
                              else
                                actionButton(
                                  title: 'Edit',
                                ),
                              actionButton(
                                title: 'Delete',
                                onTap: () {
                                  showDialog(
                                    context: context,
                                    barrierDismissible: false,
                                    builder: (BuildContext context) {
                                      return DeleteUserDialog(
                                          onTapDelete: () async {
                                        try {
                                          blocRole.add(DeleteUserEvent(
                                              user.uuid, context));
                                          await Future.delayed(
                                              const Duration(seconds: 2));
                                          return true;
                                        } catch (e) {
                                          return false;
                                        }
                                      });
                                    },
                                  ).then((v) {
                                    if (v != null) {
                                      blocRole.add(const GetUsers());
                                    }
                                  });
                                },
                              ),
                            ],
                          ),
                        ];
                      }).toList(),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.end,
                      children: [
                        SizedBox(
                          width: 500,
                          child: NumberPagination(
                            visiblePagesCount: 4,
                            buttonRadius: 10,
                            selectedButtonColor: ColorsManager.secondaryColor,
                            buttonUnSelectedBorderColor:
                                ColorsManager.grayBorder,
                            lastPageIcon:
                                const Icon(Icons.keyboard_double_arrow_right),
                            firstPageIcon:
                                const Icon(Icons.keyboard_double_arrow_left),
                            totalPages: (blocRole.totalUsersCount.length /
                                    blocRole.itemsPerPage)
                                .ceil(),
                            currentPage: blocRole.currentPage,
                            onPageChanged: (int pageNumber) {
                              context
                                  .read<UserTableBloc>()
                                  .add(ChangePage(pageNumber));
                            },
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          );
        } else if (state is ErrorState) {
          return Center(child: Text(state.message));
        } else {
          return const Center(child: Text('No data available.'));
        }
      },
    );
  }
}
