import 'package:flutter/material.dart';
import 'package:syncrow_web/common/edit_chip.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/subspace_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/button_content_widget.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/subspace_name_label_widget.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class SubSpacePartWidget extends StatefulWidget {
  SubSpacePartWidget({
    super.key,
    required this.subspaces,
    required this.onPressed,
    required this.isTagsAndSubspaceModelDisabled,
    required this.screenWidth,
    required this.editChipOnTap,
  });
  double screenWidth;
  bool isTagsAndSubspaceModelDisabled;
  final void Function() editChipOnTap;
  List<SubspaceModel>? subspaces;
  final void Function() onPressed;

  @override
  State<SubSpacePartWidget> createState() => _SubSpacePartWidgetState();
}

class _SubSpacePartWidgetState extends State<SubSpacePartWidget> {
  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        if (widget.subspaces == null || widget.subspaces!.isEmpty)
          TextButton(
            style: TextButton.styleFrom(
              padding: EdgeInsets.zero,
              overlayColor: ColorsManager.transparentColor,
            ),
            onPressed: widget.onPressed,
            child: ButtonContentWidget(
              icon: Icons.add,
              label: 'Create Sub Spaces',
              disabled: widget.isTagsAndSubspaceModelDisabled,
            ),
          )
        else
          SizedBox(
            width: widget.screenWidth * 0.25,
            child: Container(
              padding: const EdgeInsets.all(8.0),
              decoration: BoxDecoration(
                color: ColorsManager.textFieldGreyColor,
                borderRadius: BorderRadius.circular(15),
                border: Border.all(
                  color: ColorsManager.textFieldGreyColor,
                  width: 3.0, // Border width
                ),
              ),
              child: Wrap(
                spacing: 8.0,
                runSpacing: 8.0,
                children: [
                  if (widget.subspaces != null)
                    ...widget.subspaces!.map((subspace) {
                      return Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          SubspaceNameDisplayWidget(
                            text: subspace.subspaceName,
                            validateName: (updatedName) {
                              final nameExists = widget.subspaces!.any((s) {
                                final isSameId =
                                    s.internalId == subspace.internalId;
                                final isSameName =
                                    s.subspaceName.trim().toLowerCase() ==
                                        updatedName.trim().toLowerCase();

                                return !isSameId && isSameName;
                              });

                              return !nameExists;
                            },
                            onNameChanged: (updatedName) {
                              setState(() {
                                subspace.subspaceName = updatedName;
                              });
                            },
                          ),
                        ],
                      );
                    }),
                  EditChip(
                    onTap: widget.editChipOnTap,
                  )
                ],
              ),
            ),
          )
      ],
    );
  }
}
