import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/ellipsis_item_widget.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/flexible_item_widget.dart';

import 'package:syncrow_web/utils/color_manager.dart';

class DynamicProductWidget extends StatelessWidget {
  final Map<String, int> productTagCount;
  final double maxWidth;
  final double maxHeight;

  const DynamicProductWidget({
    super.key,
    required this.productTagCount,
    required this.maxWidth,
    required this.maxHeight,
  });

  @override
  Widget build(BuildContext context) {
    const itemSpacing = 8.0;
    const lineSpacing = 8.0;
    const textPadding = 16.0;
    const itemHeight = 40.0;

    final productWidgets = <Widget>[];
    var currentLineWidth = 0.0;
    var currentHeight = itemHeight;

    for (final product in productTagCount.entries) {
      final prodType = product.key;
      final count = product.value;

      final textPainter = TextPainter(
        text: TextSpan(
          text: 'x$count',
          style: Theme.of(context)
              .textTheme
              .bodySmall
              ?.copyWith(color: ColorsManager.spaceColor),
        ),
        textDirection: TextDirection.ltr,
      )..layout();

      final itemWidth = textPainter.width + textPadding + 20;

      if (currentLineWidth + itemWidth + itemSpacing > maxWidth) {
        currentHeight += itemHeight + lineSpacing;
        if (currentHeight > maxHeight) {
          productWidgets.add(const EllipsisItemWidget());
          break;
        }
        currentLineWidth = 0.0;
      }

      productWidgets.add(FlexibleItemWidget(prodType: prodType, count: count));
      currentLineWidth += itemWidth + itemSpacing;
    }

    return Wrap(
      spacing: itemSpacing,
      runSpacing: lineSpacing,
      children: productWidgets,
    );
  }
}
