import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/room_name_widget.dart';

class DynamicRoomWidget extends StatelessWidget {
  final List<SubspaceTemplateModel>? subspaceModels;
  final double maxWidth;
  final double maxHeight;

  const DynamicRoomWidget({
    super.key,
    required this.subspaceModels,
    required this.maxWidth,
    required this.maxHeight,
  });

  @override
  Widget build(BuildContext context) {
    const itemSpacing = 8.0;
    const lineSpacing = 8.0;
    const textPadding = 16.0;
    const itemHeight = 30.0;

    final roomWidgets = <Widget>[];
    var currentLineWidth = 0.0;
    var currentHeight = itemHeight;

    for (final subspace in subspaceModels!) {
      final textPainter = TextPainter(
        text: TextSpan(
            text: subspace.subspaceName,
            style: Theme.of(context).textTheme.bodyMedium),
        textDirection: TextDirection.ltr,
      )..layout();

      final itemWidth = textPainter.width + textPadding;

      if (currentLineWidth + itemWidth + itemSpacing > maxWidth) {
        currentHeight += itemHeight + lineSpacing;
        if (currentHeight > maxHeight) {
          roomWidgets.add(const RoomNameWidget(name: '...'));
          break;
        }
        currentLineWidth = 0.0;
      }

      roomWidgets.add(RoomNameWidget(name: subspace.subspaceName));
      currentLineWidth += itemWidth + itemSpacing;
    }

    return Wrap(
      spacing: itemSpacing,
      runSpacing: lineSpacing,
      children: roomWidgets,
    );
  }
}
